#!/usr/bin/python
#coding:utf-8

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: __init__.py.py
@time: 2020/6/15 10:04
"""


def main():
    pass


if __name__ == "__main__":
    main()