composite_factor = {
    'tsmom_dzq': [
        'tsmom_d3',
        'tsmom_d5'
    ],
    'tsmom_zzq': [
        'tsmom_d10',
        'tsmom_d20',
        'tsmom_d30'
    ],
    'tsmom_czq': [
        'tsmom_d60',
        'tsmom_d250',
    ],
    'xsmom_dzq': [
        'xsmom_d3_q75',
        'xsmom_d5_q75'
    ],
    'xsmom_zzq': [
        'xsmom_d10_q75',
        'xsmom_d20_q75',
        'xsmom_d30_q75'
    ],
    'xsmom_czq': [
        'xsmom_d60_q75',
        'xsmom_d250_q75',
    ],
    'qxjg': [
        'carry_d20_q70',
        'tscarry_d50',
    ],
    'tsbasismom_dzq': [
        'tsbasismom_d3',
        'tsbasismom_d5'
    ],
    'tsbasismom_zzq': [
        'tsbasismom_d10',
        'tsbasismom_d20',
        'tsbasismom_d30'
    ],
}
